﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Interactivity;
using System.Windows.Shapes;
using System.Windows.Media;
using Infragistics.Controls.Charts;

namespace ControlColumnWidth
{
    public class ColumnWidthBehavior : Behavior<ColumnSeries>
    {
        private XamDataChart Chart { get { return this.AssociatedObject.SeriesViewer as XamDataChart; } }
        private ColumnSeries Series { get { return this.AssociatedObject; } }

        public double Width
        {
            get { return (double)GetValue(WidthProperty); }
            set { SetValue(WidthProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Width.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty WidthProperty =
            DependencyProperty.Register("Width", typeof(double), typeof(ColumnWidthBehavior), new PropertyMetadata(double.NaN));

        protected override void OnAttached()
        {
            if (Chart != null)
                Chart.RefreshCompleted += Chart_RefreshCompleted;
        }

        protected override void OnDetaching()
        {
            if (Chart != null)
                Chart.RefreshCompleted -= Chart_RefreshCompleted;
        }

        void Chart_RefreshCompleted(object sender, EventArgs e)
        {
            if (Series != null && Series.RootCanvas != null && Series.RootCanvas.Children.Count > 0)
            {
                var rectangles = Series.RootCanvas.Children.Cast<FrameworkElement>().Where(x => x is Rectangle);

                double oldWidth = double.NaN;
                foreach (var rectangle in rectangles)
                {
                    oldWidth = rectangle.ActualWidth;
                    double newWidth = Width; // replace this with your desired column width

                    rectangle.Width = newWidth;
                    var transform = rectangle.RenderTransform as TranslateTransform;
                    transform.X = transform.X + ((oldWidth - newWidth) / 2);
                }
            }
        }
    }
}
